package jp.gr.java_conf.coskx.jstdemo;

import org.opencv.android.CameraBridgeViewBase.CvCameraViewFrame;
import org.opencv.core.Mat;
import org.opencv.android.CameraBridgeViewBase;
import org.opencv.android.CameraBridgeViewBase.CvCameraViewListener2;
import org.opencv.core.Point;
import org.opencv.core.Rect;
import org.opencv.core.Scalar;
import org.opencv.imgproc.Imgproc;

import android.annotation.TargetApi;
import android.app.Activity;
import android.content.pm.PackageManager;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Toast;

import static android.Manifest.permission.CAMERA;

public class MainActivity extends Activity implements CvCameraViewListener2 {
    private static final String TAG = "MainActivity";
    private static final int CAMERA_PERMISSION_REQUEST_CODE = 200;

    private CameraBridgeViewBase mOpenCvCameraView;
    private int mode =0;
    private Mat mMat;

    @Override
    protected void onStart() {
        super.onStart();
        boolean havePermission = true;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if (checkSelfPermission(CAMERA) != PackageManager.PERMISSION_GRANTED) {
                requestPermissions(new String[]{CAMERA}, CAMERA_PERMISSION_REQUEST_CODE);
                havePermission = false;
            }
        }
        if (havePermission) {
            mOpenCvCameraView.setCameraPermissionGranted();
        }
    }

    @Override
    @TargetApi(Build.VERSION_CODES.M)
    public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
        if (requestCode == CAMERA_PERMISSION_REQUEST_CODE && grantResults.length > 0
                && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
            mOpenCvCameraView.setCameraPermissionGranted();
        }
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        Log.i(TAG, "called onCreate");
        super.onCreate(savedInstanceState);
        System.loadLibrary("opencv_java4");

        setContentView(R.layout.activity_main);
        mOpenCvCameraView = (CameraBridgeViewBase) findViewById(R.id.camera_view);
        mOpenCvCameraView.setCvCameraViewListener(this);
    }

    @Override
    public void onResume() {
        super.onResume();
        mOpenCvCameraView.enableView();
        mMat = new Mat();
    }

    @Override
    public void onPause() {
        super.onPause();
        mMat.release();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        if (mOpenCvCameraView != null)
            mOpenCvCameraView.disableView();
    }

    public void onCameraViewStarted(int width, int height) { }

    public void onCameraViewStopped() { }

    //カメラからの画像はカメラの準備ができ次第，このメソッド(関数)に繰り返し送られる。
    //戻り値Matを返すと，Matが画面に表示される。
    //このメソッドに送られてくるのはinputFrameで，inputFrame.rgba()またはinputFrame.gray()でMatに変換される
    public Mat onCameraFrame(CvCameraViewFrame inputFrame) {
        if (mode == 0) {
            mMat = inputFrame.rgba(); //RGBA形式Matへの変換
        } else if (mode == 1) {
            mMat= inputFrame.gray(); //グレースケール形式Matへの変換
        } else if (mode == 2) {
            Imgproc.Canny(inputFrame.gray(), mMat, 100, 200); //grayscale canny filtering
            //inputFrame.gray()でグレースケール画像を受け取り，キャニーフィルタした値がmMatに入る
        } else if (mode == 3) {
            //青の矩形を描く
            //画像では左上が座標(0,0)で横軸は右方向，縦軸は下方向が正の方向となる
            mMat = inputFrame.rgba();
            int left = mMat.width()*4/10;//画像幅の4/10をleftにする
            int right = mMat.width()*6/10;//画像幅の6/10をrightにする
            int top = mMat.height()*4/10;//画像高さの4/10をtopにする
            int bottom = mMat.height()*6/10;//画像高さの6/10をbottomにする
            Point lt0 = new Point(left, top);//矩形左上の点の座標
            Point rb0 = new Point(right, bottom);//矩形右下の点の座標
            Imgproc.rectangle(mMat, lt0, rb0, new Scalar(2, 157, 240), 2);
            //mMatにlt0とrb0を頂点とする矩形を描いている
            //Scalar(r,g,b)は色を指定している
            //2は線の太さを指定している
        } else if (mode == 4) {
            //円を描く
            mMat = inputFrame.rgba();
            Point center = new Point(mMat.width()*4/10, mMat.height()*4/10);//中心座標緯
            int radius = mMat.width()*5/100;//半径
            Imgproc.circle(mMat,center, radius, new Scalar(250, 0, 0), 2);
            //mMatにcenterを中心とする半径radiusの円を描いている
            //Scalar(r,g,b)は色を指定している
            //2は線の太さを指定している
        } else if (mode == 5) {
            //画面左上の黒い矩形頂点を見つけて円を描く
            //ただし画面では黒い矩形の上部と左側は白いものとする
            //画像をバイト列に変換して，走査すると，最初の黒ピクセルを見つけることができる
            //mMatは表示用，mMatAnalysisは作業用
            mMat = inputFrame.rgba(); //RGBA形式Matへの変換
            Mat mMatAnalysis= inputFrame.gray(); //検出用グレースケール形式Matへの変換
            byte[] bytebuff= new byte[(int) (mMatAnalysis.total() * mMatAnalysis.channels())];
            mMatAnalysis.get(0, 0, bytebuff); //mat内のすべてのデータの取得
            //http://coskx.webcrow.jp/mrr/android/OpenCV_Mat.html　参照
            //画像は2次元データだが，一次元配列として受け取る
            //画像の最上部行のデータ行，2番目のデータ行，3番目のデータ行，・・・が全て1列につながって，1次元データとなっている
            int i;
            for (i=0; i<bytebuff.length; i++) bytebuff[i] ^= 0x80; //黒：-128　白：127　に変換
            for (i=0; i<bytebuff.length; i++) {
                if (bytebuff[i]<-64) break; //暗いピクセル発見
            }
            int width = mMatAnalysis.width();
            int y = i / width;
            int x = i % width;
            Point center = new Point(x, y);//中心座標緯
            int radius = width*5/100;//半径
            Imgproc.circle(mMat,center, radius, new Scalar(250, 0, 0), 2);
        } else if (mode == 6) {
            //指示した矩形の範囲で黒い矩形頂点を見つけて円を描く
            //ただし指示した矩形の範囲では黒い矩形の上部と左側は白いものとする
            //画像をバイト列に変換して，走査すると，最初の黒ピクセルを見つけることができる
            //mMatは表示用，それ以外のMatインスタンス(変数)は作業用
            mMat = inputFrame.rgba(); //RGBA形式Matへの変換
            Mat mMatAnalysis= inputFrame.gray(); //検出用グレースケール形式Matへの変換
            int left = mMatAnalysis.width()*4/10;//画像幅の4/10をleftにする
            int right = mMatAnalysis.width()*6/10;//画像幅の6/10をrightにする
            int top = mMatAnalysis.height()*4/10;//画像高さの4/10をtopにする
            int bottom = mMatAnalysis.height()*6/10;//画像高さの6/10をbottomにする
            Point lt0 = new Point(left, top);//矩形左上の点の座標
            Point rb0 = new Point(right, bottom);//矩形右下の点の座標
            Imgproc.rectangle(mMat, lt0, rb0, new Scalar(2, 157, 240), 2);

            Rect mRect = new Rect(lt0,rb0);//矩形を定義(青い矩形と同じ位置と大きさ)
            Mat mMatMyArea = mMatAnalysis.submat(mRect); //mMatAnalysisから矩形に切り出す
            byte[] bytebuff= new byte[(int) (mMatMyArea.total() * mMatMyArea.channels())];
            mMatMyArea.get(0, 0, bytebuff); //mat内のすべてのデータの取得
            int i;
            for (i=0; i<bytebuff.length; i++) bytebuff[i] ^= 0x80; //黒：-128　白：127　に変換
            for (i=0; i<bytebuff.length; i++) {
                if (bytebuff[i]<-64) break; //暗いピクセル発見
            }
            if (i == bytebuff.length) i = bytebuff.length-1; //見つからなかった
            int width = mMatMyArea.width();
            int y = i / width + top;
            int x = i % width + left;
            Point center = new Point(x, y);//中心座標緯
            int radius = mMatAnalysis.width() * 3/100;//半径
            Imgproc.circle(mMat,center, radius, new Scalar(250, 0, 0), 2);
        }
        return mMat;//ここでmMatを返すことによって画像がスクリーンに表示される
    }

    //スクリーンに表示されている画像をタップ（クリック）するとこのメソッドが呼ばれる
    //この仕組みはLayout/activitu_main.xmlに記述されている「android:onClick="clicedwork"」で出来ている
    public void clicedwork(View v) {
        mode++;
        //Toast.makeText(getApplicationContext(), "Test OK! mode = " + mode, Toast.LENGTH_SHORT).show();
        if (mode == 7) mode = 0;
    }
}